@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2010 by Eugene V. Lyubimkin                             *
+*   Copyright (C) 2010-2011 by Eugene V. Lyubimkin                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License                  *
@@ -23,10 +23,13 @@ using std::endl;
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <boost/lexical_cast.hpp>
+
 #include "../common.hpp"
 #include "../handlers.hpp"
 #include "../misc.hpp"
 #include "../selectors.hpp"
+#include "../colorizer.hpp"
 
 #include <cupt/system/state.hpp>
 #include <cupt/system/resolver.hpp>
@@ -37,6 +40,8 @@ using std::endl;
 #include <cupt/cache/sourceversion.hpp>
 
 typedef Worker::Action WA;
+const WA::Type fakeNotPolicyVersionAction = WA::Type(999);
+
 
 static void preProcessMode(ManagePackages::Mode& mode, const shared_ptr< Config >& config,
 		Resolver& resolver)
@@ -78,7 +83,7 @@ static void unrollFileArguments(vector< string >& arguments)
 			File file(path, "r", openError);
 			if (!openError.empty())
 			{
-				fatal("unable to open file '%s': %s", path.c_str(), openError.c_str());
+				fatal2("unable to open file '%s': %s", path, openError);
 			}
 			string line;
 			while (!file.getLine(line).eof())
@@ -219,7 +224,7 @@ static void processInstallOrRemoveExpression(const shared_ptr< const Cache >& ca
 			if (!cache->getSystemState()->getInstalledInfo(packageExpression) &&
 				!getBinaryPackage(cache, packageExpression, false))
 			{
-				fatal("unable to find binary package/expression '%s'", packageExpression.c_str());
+				fatal2("unable to find binary package/expression '%s'", packageExpression);
 			}
 
 			scheduleRemoval(packageExpression);
@@ -227,6 +232,38 @@ static void processInstallOrRemoveExpression(const shared_ptr< const Cache >& ca
 	}
 }
 
+static void processReinstallExpression(const shared_ptr< const Cache >& cache,
+		Resolver& resolver, const string& packageExpression)
+{
+	auto package = getBinaryPackage(cache, packageExpression);
+	auto installedVersion = package->getInstalledVersion();
+	if (!installedVersion)
+	{
+		fatal2("the package '%s' is not installed", packageExpression);
+	}
+	const string& installedVersionString = installedVersion->versionString;
+
+	static const string reinstallVersionSuffix = "~installed";
+	auto reinstallVersionSuffixPosition = installedVersionString.size() - reinstallVersionSuffix.size();
+	if (installedVersionString.size() > reinstallVersionSuffix.size() &&
+			installedVersionString.compare(reinstallVersionSuffixPosition, reinstallVersionSuffix.size(), reinstallVersionSuffix) == 0)
+	{
+		auto targetVersionString = installedVersionString.substr(0, reinstallVersionSuffixPosition);
+		auto targetVersion = package->getSpecificVersion(targetVersionString);
+		if (!targetVersion)
+		{
+			fatal2("the package '%s' cannot be reinstalled because there is no corresponding version (%s) available in repositories",
+					packageExpression, targetVersionString);
+		}
+		resolver.installVersion(static_pointer_cast< const BinaryVersion >(targetVersion));
+	}
+	else
+	{
+		fatal2("internal error: the installed version '%s' of the package '%s' has not a reinstall version suffix '%s'",
+				installedVersionString, packageExpression, reinstallVersionSuffix);
+	}
+}
+
 static void processPackageExpressions(const shared_ptr< Config >& config,
 		const shared_ptr< const Cache >& cache, ManagePackages::Mode& mode,
 		Resolver& resolver, const vector< string >& packageExpressions,
@@ -262,6 +299,10 @@ static void processPackageExpressions(const shared_ptr< Config >& config,
 		{
 			processBuildDependsExpression(config, cache, resolver, *packageExpressionIt);
 		}
+		else if (mode == ManagePackages::Reinstall)
+		{
+			processReinstallExpression(cache, resolver, *packageExpressionIt);
+		}
 		else
 		{
 			processInstallOrRemoveExpression(cache, resolver, mode,
@@ -281,11 +322,11 @@ void printUnpackedSizeChanges(const map< string, ssize_t >& unpackedSizesPreview
 	string message;
 	if (totalUnpackedSizeChange >= 0)
 	{
-		message = sf(__("After unpacking %s will be used."),
-				humanReadableSizeString(totalUnpackedSizeChange).c_str());
+		message = format2(__("After unpacking %s will be used."),
+				humanReadableSizeString(totalUnpackedSizeChange));
 	} else {
-		message = sf(__("After unpacking %s will be freed."),
-				humanReadableSizeString(-totalUnpackedSizeChange).c_str());
+		message = format2(__("After unpacking %s will be freed."),
+				humanReadableSizeString(-totalUnpackedSizeChange));
 	}
 
 	cout << message << endl;
@@ -295,18 +336,17 @@ void printDownloadSizes(const pair< size_t, size_t >& downloadSizes)
 {
 	auto total = downloadSizes.first;
 	auto need = downloadSizes.second;
-	cout << sf(__("Need to get %s/%s of archives. "),
-					humanReadableSizeString(need).c_str(),
-					humanReadableSizeString(total).c_str());
+	cout << format2(__("Need to get %s/%s of archives. "),
+			humanReadableSizeString(need), humanReadableSizeString(total));
 }
 
 void showVersion(const shared_ptr< const Cache >& cache, const string& packageName,
-		const Resolver::SuggestedPackage& suggestedPackage)
+		const Resolver::SuggestedPackage& suggestedPackage, WA::Type actionType)
 {
 	auto package = cache->getBinaryPackage(packageName);
 	if (!package)
 	{
-		fatal("internal error: no binary package '%s' available", packageName.c_str());
+		fatal2("internal error: no binary package '%s' available", packageName);
 	}
 	auto oldVersion = package->getInstalledVersion();
 
@@ -315,17 +355,23 @@ void showVersion(const shared_ptr< const Cache >& cache, const string& packageNa
 	auto newVersion = suggestedPackage.version;
 	string newVersionString = newVersion ? newVersion->versionString : "";
 
-	if (!oldVersionString.empty() && !newVersionString.empty())
+	if (!oldVersionString.empty() && !newVersionString.empty() &&
+			(actionType != fakeNotPolicyVersionAction || oldVersionString != newVersionString))
 	{
-		cout << sf(" [%s -> %s]", oldVersionString.c_str(), newVersionString.c_str());
+		cout << format2(" [%s -> %s]", oldVersionString, newVersionString);
 	}
 	else if (!oldVersionString.empty())
 	{
-		cout << sf(" [%s]", oldVersionString.c_str());
+		cout << format2(" [%s]", oldVersionString);
 	}
 	else
 	{
-		cout << sf(" [%s]", newVersionString.c_str());
+		cout << format2(" [%s]", newVersionString);
+	}
+
+	if (actionType == fakeNotPolicyVersionAction)
+	{
+		cout << ", " << __("preferred") << ": " << cache->getPolicyVersion(package)->versionString;
 	}
 }
 
@@ -460,7 +506,8 @@ void showReason(const Resolver::SuggestedPackage& suggestedPackage)
 	cout << endl;
 }
 
-void showUnsatisfiedSoftDependencies(const Resolver::Offer& offer)
+void showUnsatisfiedSoftDependencies(const Resolver::Offer& offer,
+		bool showDetails, std::stringstream* summaryStreamPtr)
 {
 	vector< string > messages;
 	FORIT(unresolvedProblemIt, offer.unresolvedProblems)
@@ -470,13 +517,19 @@ void showUnsatisfiedSoftDependencies(const Resolver::Offer& offer)
 
 	if (!messages.empty())
 	{
-		cout << __("Leave the following dependencies unresolved:") << endl;
-		cout << endl;
-		FORIT(messageIt, messages)
+		if (showDetails)
 		{
-			cout << *messageIt << endl;
+			cout << __("Leave the following dependencies unresolved:") << endl;
+			cout << endl;
+			FORIT(messageIt, messages)
+			{
+				cout << *messageIt << endl;
+			}
+			cout << endl;
 		}
-		cout << endl;
+
+		*summaryStreamPtr << format2(__("  %u dependency problems will stay unresolved"),
+				offer.unresolvedProblems.size()) << endl;
 	}
 }
 
@@ -514,8 +567,8 @@ Resolver::UserAnswer::Type askUserAboutSolution(
 		if (isDangerous)
 		{
 			const string confirmationForDangerousAction = __("Yes, do as I say!");
-			cout << sf(__("Dangerous actions selected. Type '%s' if you want to continue, or anything else to go back:"),
-					confirmationForDangerousAction.c_str()) << endl;
+			cout << format2(__("Dangerous actions selected. Type '%s' if you want to continue, or anything else to go back:"),
+					confirmationForDangerousAction) << endl;
 			std::getline(std::cin, answer);
 			if (answer != confirmationForDangerousAction)
 			{
@@ -550,17 +603,128 @@ Resolver::UserAnswer::Type askUserAboutSolution(
 	}
 }
 
+Resolver::SuggestedPackages generateNotPolicyVersionList(const shared_ptr< const Cache >& cache,
+		const Resolver::SuggestedPackages& packages)
+{
+	Resolver::SuggestedPackages result;
+	FORIT(suggestedPackageIt, packages)
+	{
+		const shared_ptr< const BinaryVersion >& suggestedVersion = suggestedPackageIt->second.version;
+		if (suggestedVersion)
+		{
+			auto policyVersion = cache->getPolicyVersion(getBinaryPackage(cache, suggestedVersion->packageName));
+			if (!(*policyVersion == *suggestedVersion))
+			{
+				result.insert(*suggestedPackageIt);
+			}
+		}
+	}
+
+	return result;
+}
+
+static string colorizeByActionType(const Colorizer& colorizer,
+		const string& input, WA::Type actionType, bool isAutoInstalled)
+{
+	Colorizer::Color color = Colorizer::Default;
+	switch (actionType)
+	{
+		case WA::Install: color = Colorizer::Cyan; break;
+		case WA::Remove: color = Colorizer::Yellow; break;
+		case WA::Upgrade: color = Colorizer::Green; break;
+		case WA::Purge: color = Colorizer::Red; break;
+		case WA::Downgrade: color = Colorizer::Magenta; break;
+		case WA::Configure: color = Colorizer::Blue; break;
+		default: ;
+	}
+	return colorizer.colorize(input, color, !isAutoInstalled /* bold */);
+}
+
+bool wasOrWillBePackageAutoInstalled(const Cache& cache, const string& packageName,
+		const map< string, bool >& autoFlagChanges)
+{
+	if (cache.isAutomaticallyInstalled(packageName))
+	{
+		return true;
+	}
+
+	auto autoFlagChangeIt = autoFlagChanges.find(packageName);
+	if (autoFlagChangeIt != autoFlagChanges.end())
+	{
+		if (autoFlagChangeIt->second) // "set as autoinstalled"
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+static void printPackageName(const Cache& cache, const Colorizer& colorizer,
+		const string& packageName, WA::Type actionType, const map< string, bool >& autoFlagChanges)
+{
+	bool isAutoInstalled = wasOrWillBePackageAutoInstalled(cache, packageName, autoFlagChanges);
+
+	cout << colorizeByActionType(colorizer, packageName, actionType, isAutoInstalled);
+	if (actionType == WA::Remove || actionType == WA::Purge)
+	{
+		if (isAutoInstalled && !colorizer.enabled())
+		{
+			cout << "(a)";
+		}
+	}
+}
+
+static string colorizeActionName(const Colorizer& colorizer, const string& actionName, WA::Type actionType)
+{
+	if (actionType != WA::Install && actionType != WA::Upgrade &&
+			actionType != WA::Configure && actionType != WA::ProcessTriggers)
+	{
+		return colorizer.makeBold(actionName);
+	}
+	else
+	{
+		return actionName;
+	}
+}
+
+void addActionToSummary(const Cache& cache, WA::Type actionType, const string& actionName,
+		const Resolver::SuggestedPackages& suggestedPackages, const map< string, bool >& autoFlagChanges,
+		Colorizer& colorizer, std::stringstream* summaryStreamPtr)
+{
+	size_t manuallyInstalledCount = std::count_if(suggestedPackages.begin(), suggestedPackages.end(),
+			[&cache, &autoFlagChanges](const pair< string, Resolver::SuggestedPackage >& arg)
+			{
+				return !wasOrWillBePackageAutoInstalled(cache, arg.first, autoFlagChanges);
+			});
+
+	auto total = suggestedPackages.size();
+
+	auto manualCountString = boost::lexical_cast< string >(manuallyInstalledCount);
+	auto colorizedManualCountString = colorizeByActionType(colorizer, manualCountString, actionType, false);
+
+	auto autoCountString = boost::lexical_cast< string >(total - manuallyInstalledCount);
+	auto colorizedAutoCountString = colorizeByActionType(colorizer, autoCountString, actionType, true);
+
+	*summaryStreamPtr << format2(__("  %s manually installed and %s automatically installed packages %s"),
+			colorizedManualCountString, colorizedAutoCountString, actionName) << endl;
+}
+
 Resolver::CallbackType generateManagementPrompt(const shared_ptr< const Config >& config,
 		const shared_ptr< const Cache >& cache, const shared_ptr< Worker >& worker,
-		bool showVersions, bool showSizeChanges, const set< string >& purgedPackageNames,
-		bool& addArgumentsFlag)
+		bool showVersions, bool showSizeChanges, bool showNotPreferred,
+		const set< string >& purgedPackageNames, bool& addArgumentsFlag, bool& thereIsNothingToDo)
 {
-	auto result = [&config, &cache, &worker, showVersions, showSizeChanges, &purgedPackageNames, &addArgumentsFlag]
+	auto result = [&config, &cache, &worker, showVersions, showSizeChanges, showNotPreferred,
+			&purgedPackageNames, &addArgumentsFlag, &thereIsNothingToDo]
 			(const Resolver::Offer& offer) -> Resolver::UserAnswer::Type
 	{
 		addArgumentsFlag = false;
+		thereIsNothingToDo = false;
 
 		auto showReasons = config->getBool("cupt::resolver::track-reasons");
+		auto showSummary = config->getBool("cupt::console::actions-preview::show-summary");
+		auto showDetails = config->getBool("cupt::console::actions-preview::show-details");
 
 		worker->setDesiredState(offer);
 		FORIT(packageNameIt, purgedPackageNames)
@@ -571,55 +735,71 @@ Resolver::CallbackType generateManagementPrompt(const shared_ptr< const Config >
 		auto actionsPreview = worker->getActionsPreview();
 		auto unpackedSizesPreview = worker->getUnpackedSizesPreview();
 
+		Colorizer colorizer(*config);
+
+		std::stringstream summaryStream;
+
 		size_t actionCount = 0;
 		{ // print planned actions
-			static const map< WA::Type, string > actionNames = {
-				{ WA::Install, __("INSTALLED") },
-				{ WA::Remove, __("REMOVED") },
-				{ WA::Upgrade, __("UPGRADED") },
-				{ WA::Purge, __("PURGED") },
-				{ WA::Downgrade, __("DOWNGRADED") },
-				{ WA::Configure, __("CONFIGURED") },
-				{ WA::Deconfigure, __("DECONFIGURED") },
-				{ WA::ProcessTriggers, __("TRIGGER-PROCESSED") },
+			const map< WA::Type, string > actionNames = {
+				{ WA::Install, __("will be installed") },
+				{ WA::Remove, __("will be removed") },
+				{ WA::Upgrade, __("will be upgraded") },
+				{ WA::Purge, __("will be purged") },
+				{ WA::Downgrade, __("will be downgraded") },
+				{ WA::Configure, __("will be configured") },
+				{ WA::Deconfigure, __("will be deconfigured") },
+				{ WA::ProcessTriggers, __("will have triggers processed") },
+				{ fakeNotPolicyVersionAction, __("will have a not preferred version") },
 			};
 			cout << endl;
 
-			static const WA::Type actionTypesInOrder[] = { WA::Install, WA::Upgrade, WA::Remove,
+			vector< WA::Type > actionTypesInOrder = { WA::Install, WA::Upgrade, WA::Remove,
 					WA::Purge, WA::Downgrade, WA::Configure, WA::ProcessTriggers, WA::Deconfigure };
-
-			for (size_t i = 0; i < sizeof(actionTypesInOrder) / sizeof(WA::Type); ++i)
+			if (showNotPreferred)
 			{
-				const WA::Type& actionType = actionTypesInOrder[i];
-
-				const Resolver::SuggestedPackages& actionSuggestedPackages = actionsPreview->groups[actionType];
+				actionTypesInOrder.push_back(fakeNotPolicyVersionAction);
+			}
 
+			FORIT (actionTypeIt, actionTypesInOrder)
+			{
+				const WA::Type& actionType = *actionTypeIt;
+				const Resolver::SuggestedPackages& actionSuggestedPackages =
+						actionType == fakeNotPolicyVersionAction ?
+						generateNotPolicyVersionList(cache, offer.suggestedPackages) : actionsPreview->groups[actionType];
 				if (actionSuggestedPackages.empty())
 				{
-					continue; // don't print empty lists
+					continue;
+				}
+
+				if (actionType != fakeNotPolicyVersionAction)
+				{
+					actionCount += actionSuggestedPackages.size();
 				}
 
 				const string& actionName = actionNames.find(actionType)->second;
-				cout << sf(__("The following %u packages will be %s:"),
-						actionSuggestedPackages.size(), actionName.c_str()) << endl << endl;
+
+				if (showSummary)
+				{
+					addActionToSummary(*cache, actionType, actionName, actionSuggestedPackages,
+							actionsPreview->autoFlagChanges, colorizer, &summaryStream);
+				}
+				if (!showDetails)
+				{
+					continue;
+				}
+
+				cout << format2(__("The following packages %s:"),
+						colorizeActionName(colorizer, actionName, actionType)) << endl << endl;
 
 				FORIT(it, actionSuggestedPackages)
 				{
-					++actionCount;
-
 					const string& packageName = it->first;
-					cout << packageName;
-					if (actionType == WA::Remove || actionType == WA::Purge)
-					{
-						if (cache->isAutomaticallyInstalled(packageName))
-						{
-							cout << "(a)";
-						}
-					}
+					printPackageName(*cache, colorizer, packageName, actionType, actionsPreview->autoFlagChanges);
 
 					if (showVersions)
 					{
-						showVersion(cache, packageName, it->second);
+						showVersion(cache, packageName, it->second, actionType);
 					}
 
 					if (showSizeChanges)
@@ -647,15 +827,22 @@ Resolver::CallbackType generateManagementPrompt(const shared_ptr< const Config >
 				}
 				cout << endl;
 			}
+
+			showUnsatisfiedSoftDependencies(offer, showDetails, &summaryStream);
 		};
 
 		// nothing to do maybe?
 		if (actionCount == 0)
 		{
-			return Resolver::UserAnswer::Accept;
+			thereIsNothingToDo = true;
+			return Resolver::UserAnswer::Abandon;
 		}
 
-		showUnsatisfiedSoftDependencies(offer);
+		if (showSummary)
+		{
+			cout << __("Action summary:") << endl << summaryStream.str() << endl;
+			summaryStream.clear();
+		}
 
 		bool isDangerousAction = false;
 		if (!config->getBool("cupt::console::allow-untrusted"))
@@ -681,8 +868,9 @@ Resolver::CallbackType generateManagementPrompt(const shared_ptr< const Config >
 	return result;
 }
 
-void parseManagementOptions(Context& context, vector< string >& packageExpressions,
-		bool& showVersions, bool& showSizeChanges)
+void parseManagementOptions(Context& context, ManagePackages::Mode mode,
+		vector< string >& packageExpressions,
+		bool& showVersions, bool& showSizeChanges, bool& showNotPreferred)
 {
 	bpo::options_description options;
 	options.add_options()
@@ -695,7 +883,10 @@ void parseManagementOptions(Context& context, vector< string >& packageExpressio
 		("show-size-changes,Z", "")
 		("show-reasons,D", "")
 		("show-deps", "")
+		("show-not-preferred", "")
 		("download-only,d", "")
+		("summary-only", "")
+		("no-summary", "")
 		("assume-yes", "")
 		("yes,y", "");
 
@@ -750,9 +941,24 @@ void parseManagementOptions(Context& context, vector< string >& packageExpressio
 	{
 		config->setScalar("cupt::resolver::auto-remove", "no");
 	}
+	if (variables.count("summary-only"))
+	{
+		config->setScalar("cupt::console::actions-preview::show-summary", "yes");
+		config->setScalar("cupt::console::actions-preview::show-details", "no");
+	}
+	if (variables.count("no-summary"))
+	{
+		config->setScalar("cupt::console::actions-preview::show-summary", "no");
+		config->setScalar("cupt::console::actions-preview::show-details", "yes");
+	}
 
 	showVersions = variables.count("show-versions");
 	showSizeChanges = variables.count("show-size-changes");
+	string showNotPreferredConfigValue = config->getString("cupt::console::actions-preview::show-not-preferred");
+	showNotPreferred = variables.count("show-not-preferred") ||
+			showNotPreferredConfigValue == "yes" ||
+			((mode == ManagePackages::FullUpgrade || mode == ManagePackages::SafeUpgrade) &&
+					showNotPreferredConfigValue == "for-upgrades");
 }
 
 int managePackages(Context& context, ManagePackages::Mode mode)
@@ -769,9 +975,9 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 	Cache::memoize = true;
 
 	vector< string > packageExpressions;
-	bool showVersions;
-	bool showSizeChanges;
-	parseManagementOptions(context, packageExpressions, showVersions, showSizeChanges);
+	bool showVersions, showSizeChanges, showNotPreferred;
+	parseManagementOptions(context, mode, packageExpressions,
+			showVersions, showSizeChanges, showNotPreferred);
 
 	unrollFileArguments(packageExpressions);
 
@@ -781,7 +987,7 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 	{
 		if (packageExpressions.size() != 1)
 		{
-			fatal("exactly one argument (the snapshot name) should be specified");
+			fatal2("exactly one argument (the snapshot name) should be specified");
 		}
 		snapshotName = packageExpressions[0];
 		packageExpressions.clear();
@@ -797,7 +1003,6 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 		vector< string > packageNameGlobsToReinstall;
 		if (mode == ManagePackages::Reinstall)
 		{
-			mode = ManagePackages::Install;
 			packageNameGlobsToReinstall = packageExpressions;
 		}
 
@@ -814,7 +1019,7 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 	{
 		if (!config->getString("cupt::resolver::external-command").empty())
 		{
-			fatal("using external resolver is not supported now");
+			fatal2("using external resolver is not supported now");
 		}
 		else
 		{
@@ -839,9 +1044,10 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 
 	cout << __("Resolving possible unmet dependencies... ") << endl;
 
-	bool addArgumentsFlag;
+	bool addArgumentsFlag, thereIsNothingToDo;
 	auto callback = generateManagementPrompt(config, cache, worker,
-			showVersions, showSizeChanges, purgedPackageNames, addArgumentsFlag);
+			showVersions, showSizeChanges, showNotPreferred,
+			purgedPackageNames, addArgumentsFlag, thereIsNothingToDo);
 
 	resolve:
 	bool resolved = resolver->resolve(callback);
@@ -868,7 +1074,12 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 	// at this stage resolver has done its work, so to does not consume the RAM
 	resolver.reset();
 
-	if (resolved)
+	if (thereIsNothingToDo)
+	{
+		cout << __("Nothing to do.") << endl;
+		return 0;
+	}
+	else if (resolved)
 	{
 		// if some solution was found and user has accepted it
 
@@ -880,7 +1091,7 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 		}
 		catch (Exception&)
 		{
-			fatal("unable to do requested actions");
+			fatal2("unable to do requested actions");
 		}
 		return 0;
 	}
@@ -893,6 +1104,11 @@ int managePackages(Context& context, ManagePackages::Mode mode)
 
 int distUpgrade(Context& context)
 {
+	if (shellMode)
+	{
+		fatal2("'dist-upgrade' command cannot be run in the shell mode");
+	}
+
 	{ // 1st stage: upgrading of package management tools
 		cout << __("[ upgrading package management tools ]") << endl;
 		cout << endl;
@@ -900,7 +1116,7 @@ int distUpgrade(Context& context)
 		context.unparsed.push_back("cupt");
 		if (managePackages(context, ManagePackages::Install) != 0)
 		{
-			fatal("upgrading of the package management tools failed");
+			fatal2("upgrading of the package management tools failed");
 		}
 	}
 
@@ -918,7 +1134,7 @@ int distUpgrade(Context& context)
 			}
 		}
 		execvp(argv[0], argv);
-		fatal("upgrading the system failed: execvp failed: EEE");
+		fatal2e("upgrading the system failed: execvp failed");
 	}
 	return 0; // unreachable due to exec
 }
@@ -995,17 +1211,17 @@ int cleanArchives(Context& context, bool leaveAvailable)
 		struct stat stat_structure;
 		if (lstat(path.c_str(), &stat_structure))
 		{
-			fatal("lstat() failed: '%s': EEE", path.c_str());
+			fatal2e("lstat() failed: '%s'", path);
 		}
 		else
 		{
 			size_t size = stat_structure.st_size;
 			totalDeletedBytes += size;
-			cout << sf(__("Deleting '%s' <%s>..."), path.c_str(), humanReadableSizeString(size).c_str()) << endl;
+			cout << format2(__("Deleting '%s' <%s>..."), path, humanReadableSizeString(size)) << endl;
 			worker.deleteArchive(path);
 		}
 	}
-	cout << sf(__("Freed %s of disk space."), humanReadableSizeString(totalDeletedBytes).c_str()) << endl;
+	cout << format2(__("Freed %s of disk space."), humanReadableSizeString(totalDeletedBytes)) << endl;
 
 	cout << __("Deleting partial archives...") << endl;
 	worker.deletePartialArchives();
