@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2010 by Eugene V. Lyubimkin                             *
+*   Copyright (C) 2010-2011 by Eugene V. Lyubimkin                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License                  *
@@ -223,7 +223,7 @@ void CacheImpl::parseSourcesLists()
 	}
 	catch (Exception&)
 	{
-		fatal("error while parsing sources list");
+		fatal2("error while parsing sources list");
 	}
 }
 
@@ -233,7 +233,7 @@ void CacheImpl::parseSourceList(const string& path)
 	File file(path, "r", openError);
 	if (!openError.empty())
 	{
-		fatal("unable to open file '%s': %s", path.c_str(), openError.c_str());
+		fatal2("unable to open file '%s': %s", path, openError);
 	}
 
 	string line;
@@ -258,7 +258,7 @@ void CacheImpl::parseSourceList(const string& path)
 			// type
 			if (tokens.empty())
 			{
-				fatal("undefined source type at file '%s', line %u", path.c_str(), lineNumber);
+				fatal2("undefined source type at file '%s', line %u", path, lineNumber);
 			}
 			else
 			{
@@ -272,14 +272,14 @@ void CacheImpl::parseSourceList(const string& path)
 				}
 				else
 				{
-					fatal("incorrect source type at file '%s', line %u", path.c_str(), lineNumber);
+					fatal2("incorrect source type at file '%s', line %u", path, lineNumber);
 				}
 			}
 
 			// uri
 			if (tokens.size() < 2)
 			{
-				fatal("undefined source uri at file '%s', line %u", path.c_str(), lineNumber);
+				fatal2("undefined source uri at file '%s', line %u", path, lineNumber);
 			}
 			else
 			{
@@ -288,7 +288,7 @@ void CacheImpl::parseSourceList(const string& path)
 
 			if (tokens.size() < 3)
 			{
-				fatal("undefined source distribution at file '%s', line %u", path.c_str(), lineNumber);
+				fatal2("undefined source distribution at file '%s', line %u", path, lineNumber);
 			}
 			else
 			{
@@ -321,14 +321,105 @@ void CacheImpl::parseSourceList(const string& path)
 				}
 				else
 				{
-					fatal("distribution doesn't end with a slash at file '%s', line %u", path.c_str(), lineNumber);
+					fatal2("distribution doesn't end with a slash at file '%s', line %u", path, lineNumber);
 				}
 			}
 		}
 	}
 }
 
-void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
+class ReleaseLimits
+{
+	struct Item
+	{
+		std::function< string (const ReleaseInfo&) > attributeExtractor;
+		bool typeIsInclude;
+		vector< string > values;
+	};
+	std::list< Item > __items;
+ public:
+	ReleaseLimits(const Config& config)
+	{
+		const static map< string, std::function< string (const ReleaseInfo&) > > limitCategories = {
+			{ "archive", [](const ReleaseInfo& ri) { return ri.archive; } },
+			{ "codename", [](const ReleaseInfo& ri) { return ri.codename; } }
+		};
+
+		FORIT(categoryIt, limitCategories)
+		{
+			auto limitValuesOptionName = string("cupt::cache::limit-releases::by-") + categoryIt->first;
+			auto limitTypeOptionName = limitValuesOptionName + "::type";
+
+			auto limitType = config.getString(limitTypeOptionName);
+			bool limitTypeIsInclude = true /* will be re-initialized anyway */;
+			if (limitType == "none")
+			{
+				continue;
+			}
+			else if (limitType == "include")
+			{
+				limitTypeIsInclude = true;
+			}
+			else if (limitType == "exclude")
+			{
+				limitTypeIsInclude = false;
+			}
+			else
+			{
+				try
+				{
+					fatal2("the option '%s' can have only values 'none', 'include' or 'exclude'",
+							limitTypeOptionName);
+				}
+				catch (Exception&)
+				{
+					continue;
+				}
+			}
+			auto limitValues = config.getList(limitValuesOptionName);
+
+			__items.push_back(Item { categoryIt->second, limitTypeIsInclude, limitValues });
+		}
+	}
+
+	bool isExcluded(const ReleaseInfo& releaseInfo) const
+	{
+		FORIT(itemIt, __items)
+		{
+			bool foundInLimitValues = std::find(itemIt->values.begin(), itemIt->values.end(),
+					(itemIt->attributeExtractor)(releaseInfo)) != itemIt->values.end();
+			if (itemIt->typeIsInclude != foundInLimitValues)
+			{
+				return true;
+			}
+		}
+
+		return false;
+	}
+};
+
+void CacheImpl::processIndexEntries(bool useBinary, bool useSource)
+{
+	ReleaseLimits releaseLimits(*config);
+	FORIT(indexEntryIt, indexEntries)
+	{
+		const IndexEntry& entry = *indexEntryIt;
+
+		if (entry.category == IndexEntry::Binary && !useBinary)
+		{
+			continue;
+		}
+		if (entry.category == IndexEntry::Source && !useSource)
+		{
+			continue;
+		}
+
+		processIndexEntry(entry, releaseLimits);
+	}
+}
+
+void CacheImpl::processIndexEntry(const IndexEntry& indexEntry,
+		const ReleaseLimits& releaseLimits)
 {
 	string indexFileToParse = cachefiles::getPathOfIndexList(*config, indexEntry);
 
@@ -344,6 +435,11 @@ void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
 		releaseInfo->baseUri = indexEntry.uri;
 		releaseInfo->verified = cachefiles::verifySignature(*config, releaseFilePath);
 
+		if (releaseLimits.isExcluded(*releaseInfo))
+		{
+			return;
+		}
+
 		if (indexEntry.category == IndexEntry::Binary)
 		{
 			binaryReleaseData.push_back(releaseInfo);
@@ -357,7 +453,7 @@ void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
 	}
 	catch (Exception&)
 	{
-		warn("skipped the index '%s'", indexAlias.c_str());
+		warn2("skipped the index '%s'", indexAlias);
 	}
 
 	try  // processing translations if any
@@ -371,13 +467,12 @@ void CacheImpl::processIndexEntry(const IndexEntry& indexEntry)
 			if (errorString.empty())
 			{
 				processTranslationFile(*pathIt);
-				break;
 			}
 		}
 	}
 	catch (Exception&)
 	{
-		warn("skipped translations of the index '%s'", indexAlias.c_str());
+		warn2("skipped translations of the index '%s'", indexAlias);
 	}
 }
 
@@ -394,7 +489,7 @@ void CacheImpl::processIndexFile(const string& path, IndexEntry::Type category,
 	shared_ptr< File > file(new File(path, "r", openError));
 	if (!openError.empty())
 	{
-		fatal("unable to open index file '%s': %s", path.c_str(), openError.c_str());
+		fatal2("unable to open index file '%s': %s", path, openError);
 	}
 
 	releaseInfoAndFileStorage.push_back(make_pair(releaseInfo, file));
@@ -423,7 +518,7 @@ void CacheImpl::processIndexFile(const string& path, IndexEntry::Type category,
 			}
 			else
 			{
-				fatal("unable to find correct Package line");
+				fatal2("unable to find correct Package line");
 			}
 
 			try
@@ -432,7 +527,7 @@ void CacheImpl::processIndexFile(const string& path, IndexEntry::Type category,
 			}
 			catch (Exception&)
 			{
-				warn("discarding this package version from index file '%s'", path.c_str());
+				warn2("discarding this package version from index file '%s'", path);
 				while (file->rawGetLine(buf, size), size > 1) {}
 				continue;
 			}
@@ -454,7 +549,7 @@ void CacheImpl::processIndexFile(const string& path, IndexEntry::Type category,
 	}
 	catch (Exception&)
 	{
-		fatal("error parsing index file '%s'", path.c_str());
+		fatal2("error parsing index file '%s'", path);
 	}
 }
 
@@ -464,7 +559,7 @@ void CacheImpl::processTranslationFile(const string& path)
 	shared_ptr< File > file(new File(path, "r", errorString));
 	if (!errorString.empty())
 	{
-		fatal("unable to open translation file '%s': %s", path.c_str(), errorString.c_str());
+		fatal2("unable to open translation file '%s': %s", path, errorString);
 	}
 
 	try
@@ -503,19 +598,19 @@ void CacheImpl::processTranslationFile(const string& path)
 
 			if (!hashSumFound)
 			{
-				fatal("unable to find md5 hash in a translation record starting at byte '%u'", recordPosition);
+				fatal2("unable to find md5 hash in a translation record starting at byte '%u'", recordPosition);
 			}
 			if (!translationFound)
 			{
-				fatal("unable to find translation in a translation record starting at byte '%u'", recordPosition);
+				fatal2("unable to find translation in a translation record starting at byte '%u'", recordPosition);
 			}
 
-			translations[md5] = translationPosition;
+			translations.insert(make_pair(std::move(md5), translationPosition));
 		}
 	}
 	catch(Exception&)
 	{
-		fatal("error parsing translation file '%s'", path.c_str());
+		fatal2("error parsing translation file '%s'", path);
 	}
 }
 
@@ -581,7 +676,7 @@ void CacheImpl::parseExtendedStates()
 		File file(path, "r", openError);
 		if (!openError.empty())
 		{
-			fatal("unable to open file '%s': %s", path.c_str(), openError.c_str());
+			fatal2("unable to open file '%s': %s", path, openError);
 		}
 
 		internal::TagParser parser(&file);
@@ -591,8 +686,7 @@ void CacheImpl::parseExtendedStates()
 		{
 			if (!tagName.equal("Package", 7))
 			{
-				fatal("wrong tag: expected 'Package', got '%s' at file '%s'",
-						string(tagName).c_str(), path.c_str());
+				fatal2("wrong tag: expected 'Package', got '%s' at file '%s'", string(tagName), path);
 			}
 
 			string packageName = tagValue;
@@ -610,22 +704,21 @@ void CacheImpl::parseExtendedStates()
 					}
 					else if (!tagValue.equal(BUFFER_AND_SIZE("0")))
 					{
-						fatal("bad value '%s' (should be 0 or 1) for the package '%s' at file '%s'",
-								string(tagValue).c_str(), packageName.c_str(), path.c_str());
+						fatal2("bad value '%s' (should be 0 or 1) for the package '%s' at file '%s'",
+								string(tagValue), packageName, path);
 					}
 				}
 			}
 
 			if (!valueFound)
 			{
-				fatal("no 'Auto-Installed' tag for the package '%s' at file '%s'",
-						packageName.c_str(), path.c_str());
+				fatal2("no 'Auto-Installed' tag for the package '%s' at file '%s'", packageName, path);
 			}
 		}
 	}
 	catch (Exception&)
 	{
-		fatal("error while parsing extended states");
+		fatal2("error while parsing extended states");
 	}
 }
 
