@@ -1,5 +1,5 @@
 /**************************************************************************
-*   Copyright (C) 2010 by Eugene V. Lyubimkin                             *
+*   Copyright (C) 2010-2011 by Eugene V. Lyubimkin                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License                  *
@@ -28,7 +28,7 @@ namespace internal {
 
 WorkerBase::WorkerBase()
 {
-	fatal("internal error: WorkerBase::WorkerBase shouldn't be ever called");
+	fatal2("internal error: WorkerBase::WorkerBase shouldn't be ever called");
 }
 
 WorkerBase::WorkerBase(const shared_ptr< const Config >& config, const shared_ptr< const Cache >& cache)
@@ -63,23 +63,24 @@ string WorkerBase::_get_archive_basename(const shared_ptr< const BinaryVersion >
 void WorkerBase::_run_external_command(Logger::Subsystem subsystem,
 		const string& command, const string& commandInput, const string& errorId)
 {
-	_logger->log(subsystem, 3, sf("running: %s", command.c_str()));
+	const Logger::Level level = 3;
+	_logger->log(subsystem, level, format2("running: %s", command));
 
 	if (_config->getBool("cupt::worker::simulate"))
 	{
 		if (commandInput.empty())
 		{
-			simulate("running command '%s'", command.c_str());
+			simulate2("running command '%s'", command);
 		}
 		else
 		{
-			simulate("running command '%s' with the input\n-8<-\n%s\n->8-",
-					command.c_str(), commandInput.c_str());
+			simulate2("running command '%s' with the input\n-8<-\n%s\n->8-",
+					command, commandInput);
 		}
 	}
 	else
 	{
-		const char* id = (errorId.empty() ? command.c_str() : errorId.c_str());
+		const string& id = (errorId.empty() ? command : errorId);
 
 		if (commandInput.empty())
 		{
@@ -87,13 +88,13 @@ void WorkerBase::_run_external_command(Logger::Subsystem subsystem,
 			auto result = ::system(command.c_str());
 			if (result == -1)
 			{
-				fatal("unable to launch command '%s': EEE",
-						command.c_str());
+				_logger->loggedFatal(subsystem, level,
+						format2e("unable to launch command '%s'", command));
 			}
 			else if (result)
 			{
-				fatal("command '%s' execution failed: %s", command.c_str(),
-						getWaitStatusDescription(result).c_str());
+				_logger->loggedFatal(subsystem, level,
+						format2("command '%s' execution failed: %s", command, getWaitStatusDescription(result)));
 			}
 		}
 		else try
@@ -103,15 +104,15 @@ void WorkerBase::_run_external_command(Logger::Subsystem subsystem,
 			File pipeFile(command, "pw", errorString);
 			if (!errorString.empty())
 			{
-				fatal("unable to launch a pipe to the command '%s': %s",
-						command.c_str(), errorString.c_str());
+				_logger->loggedFatal(subsystem, level,
+						format2("unable to launch a pipe to the command '%s': %s", command, errorString));
 			}
 
 			pipeFile.put(commandInput);
 		}
 		catch (...)
 		{
-			fatal("%s failed", id);
+			_logger->loggedFatal(subsystem, level, id + " failed");
 		}
 	}
 }
